use std::net::UdpSocket;
use std::str;
use std::thread;
use std::env;

fn main() {
    let args: Vec<String> = env::args().collect();
    if args.len() != 3 {
        println!("Usage: {} [ip] [port]", args[0]);
        return;
    }

    let ip = ::std::env::args().nth(1).unwrap();
    let port = ::std::env::args().nth(2).unwrap();
    let my_bind = ip.to_string() + ":" + &port;
    let my_bind_final: &str = &my_bind;

    let socket = match UdpSocket::bind(my_bind_final) {
        Ok(s) => s,
        Err(e) => panic!("Socket binding failed: {}", e)
    };

    let mut buf = [0; 128];
    loop {
        let sock = socket.try_clone().expect("Failed to clone socket!");
        match socket.recv_from(&mut buf) {
            Ok((amt, src)) => {
                thread::spawn(move || {
                    println!("Got {} bytes from {}.", amt, src);
                    print!("{}", str::from_utf8(&buf[0..amt]).unwrap_or(""));
                    sock.send_to(&buf, &src).expect("Failed to send a response");
                });
            },
            Err(e) => {
                println!("Failed to receive data: {}", e);
            }
        }
    }
}
